@@ -8,6 +8,7 @@ from typing import List, TypeVar, Tuple, Dict
 import numpy as np
 import pandas as pd
 from IPython.display import display
+from ansys.mapdl.core import LocalMapdlPool
 
 from magnumapi.geometry.Geometry import Geometry
 from magnumapi.geometry.GeometryChange import GeometryChange
@@ -15,8 +16,11 @@ from magnumapi.optimization.DesignVariable import GeneticDesignVariable
 from magnumapi.optimization.Logger import Logger
 from magnumapi.optimization.ModelExecutor import ModelExecutorFactory
 from magnumapi.optimization.OptimizationConfig import OptimizationConfig
+from magnumapi.optimization.ParallelOptimizationConfig import ParallelOptimizationConfig
 from magnumapi.optimization.OptimizationNotebookConfig import OptimizationNotebookConfig
 from magnumapi.optimization.constants import PENALTY, ERROR_KEY
+from magnumapi.tool_adapters.ansys.AnsysInputBuilder import AnsysInputBuilder
+from magnumapi.tool_adapters.ansys.AnsysToolAdapter import AnsysToolAdapter
 
 T = TypeVar('T')
 
@@ -44,22 +48,20 @@ class Optimization(ABC):
         for gen in range(self.n_gen):
             print('Generation:', gen)
             # decode population
-            scores = []
+            fom_dcts = []
 
             for index, individual in enumerate(self.pop):
                 print('\tIndividual:', index)
                 try:
                     self.update_model(individual)
-
-                    fom_dct = self.calculate_figures_of_merit(individual)
-                    score = self.calculate_score(fom_dct)
-                    scores.append(score)
+                    fom_dct = self.calculate_figures_of_merit(individual, {})
                 except AttributeError:
                     fom_dct = {objective_config.objective: float('nan') for objective_config in self.config.objectives}
-                    score = PENALTY
-                    scores.append(score)
+                fom_dcts.append(fom_dct)
 
-                # log current computation results
+            # log current computation results
+            scores = self.calculate_scores(fom_dcts)
+            for individual, fom_dct, score in zip(self.pop, fom_dcts, scores):
                 self.logger.append_to_logger(self.decode_individual(individual), fom_dct, score)
 
             # display an update
@@ -98,10 +100,11 @@ class Optimization(ABC):
         raise NotImplementedError('This method is not implemented for this class')
 
     @abstractmethod
-    def calculate_figures_of_merit(self, individual: List[int]) -> dict:
+    def calculate_figures_of_merit(self, individual: List[int], fom_init: dict={}) -> dict:
         """ Abstract method calculating figures of merit
 
         :param individual: a list of 0s and 1s representing a model to be optimised
+        :param fom_init: an initial value of the figure of merit dictionary
         """
         raise NotImplementedError('This method is not implemented for this class')
 
@@ -115,6 +118,9 @@ class Optimization(ABC):
         """
         raise NotImplementedError('This method is not implemented for this class')
 
+    def calculate_scores(self, fom_dcts: list) -> list:
+        return [self.calculate_score(fom_dct) for fom_dct in fom_dcts]
+
     @abstractmethod
     def update_generation(self, pop: List[List[int]], scores: List[float], k_selection=3) -> List[List[int]]:
         raise NotImplementedError('This method is not implemented for this class')
@@ -156,13 +162,14 @@ class GeneticOptimization(Optimization, ABC):
     @staticmethod
     def initialize_design_variables(design_variables_df: pd.DataFrame) -> List[GeneticDesignVariable]:
         """ Static method initializing design variables from a DataFrame
+        # todo: add description of columns in the dataframe
 
         :param design_variables_df: a DataFrame with design variables
         :return: a list of design variables
         """
         design_variables = []
         for index, row in design_variables_df.iterrows():
-            design_variables.append(GeneticDesignVariable(**row.to_dict()))
+                design_variables.append(GeneticDesignVariable(**row.to_dict()))
 
         return design_variables
 
@@ -210,7 +217,7 @@ class GeneticOptimization(Optimization, ABC):
         selected = [GeneticOptimization.selection(pop_sorted, scores_sorted, k_selection)
                     for _ in range(self.n_pop - self.n_elite)]
         # create the next generation
-        children = list()
+        children = []
         for i in range(0, self.n_pop - self.n_elite, 2):
             # get selected parents in pairs
             p1, p2 = selected[i], selected[i + 1]
@@ -309,11 +316,11 @@ class RoxieGeneticOptimization(GeneticOptimization):
         self.is_script_executed = is_script_executed
 
     def initialize_population(self) -> List[List[int]]:
-        pop = []
+        population = []
         for _ in range(self.n_pop):
-            chromosome = RoxieGeneticOptimization.generate_random_chromosome_programmable_geometry(self.design_variables)
-            pop.append(chromosome)
-        return pop
+            ind = RoxieGeneticOptimization.generate_random_chromosome_programmable_geometry(self.design_variables)
+            population.append(ind)
+        return population
 
     def initialize_population_old(self) -> List[List[int]]:
         pop = []
@@ -339,6 +346,11 @@ class RoxieGeneticOptimization(GeneticOptimization):
     def generate_random_chromosome_programmable_geometry(design_variables):
         n_layers = max([dv.layer for dv in design_variables if 'nbl_layer' in dv.variable])
         chromosome = []
+
+        if any(['r_aperture' == dv.variable for dv in design_variables]):
+            r_ap_dv = [dv for dv in design_variables if dv.variable == 'r_aperture'][0]
+            chromosome.extend(r_ap_dv.generate_random_gene())
+
         for layer_index in range(1, n_layers + 1):
             # randomly select spar thickness
             st_dv = [dv for dv in design_variables if dv.variable == 'spar_thickness' and dv.layer == layer_index][0]
@@ -387,6 +399,9 @@ class RoxieGeneticOptimization(GeneticOptimization):
         with open(self.model_input_path, 'w') as file:
             json.dump(updated_block_inputs, file, sort_keys=True, indent=4)
 
+    def decode_individual(self, individual: List[int]) -> Dict:
+        return self.decode_chromosome_programmable_geometry(individual)
+
     def decode_individual_old(self, individual: List[int]) -> Dict:
         block_variable_value = {}
         for index, design_variable in enumerate(self.design_variables):
@@ -401,13 +416,17 @@ class RoxieGeneticOptimization(GeneticOptimization):
 
         return block_variable_value
 
-    def decode_individual(self, individual: List[int]) -> Dict:
-        return self.decode_chromosome_programmable_geometry(individual)
-
     def decode_chromosome_programmable_geometry(self, individual: List[int]):
         decoded = {}
         # remove nbl_layer and nco_layer from the list of design variables
         index_start = 0
+        if any(['r_aperture' == dv.variable for dv in self.design_variables]):
+            r_ap_dv = [dv for dv in self.design_variables if dv.variable == 'r_aperture'][0]
+            index_end = index_start + r_ap_dv.bits
+            value = r_ap_dv.convert_gene_to_value(individual[index_start:index_end])
+            index_start = index_end
+            decoded['r_aperture'] = value
+
         n_layers = max([dv.layer for dv in self.design_variables if 'nbl_layer' in dv.variable])
         for layer_index in range(1, n_layers + 1):
             nbl_layer_dv = [dv for dv in self.design_variables
@@ -458,13 +477,16 @@ class RoxieGeneticOptimization(GeneticOptimization):
         :return: an updated list of block definitions
         """
         geometry_model = deepcopy(self.geometry)
+        if 'r_aperture' in decoded_chromosome.keys():
+            geometry_model.r_aperture = decoded_chromosome['r_aperture']
+
         for block_variable, value in decoded_chromosome.items():
             if ':-:' in block_variable:
                 variable, layer_index, block_index = block_variable.split(':')
                 block_index = int(block_index)
                 if variable == 'current':
                     geometry_model.blocks[block_index - 1].block_def.current = value
-            elif ':nan:' not in block_variable:
+            elif ':' in block_variable:
                 variable, layer_index, block_layer_index = block_variable.split(':')
                 layer_index = int(layer_index)
                 block_layer_index = int(block_layer_index)
@@ -531,7 +553,7 @@ class RoxieGeneticOptimization(GeneticOptimization):
 
         return geometry_rel
 
-    def calculate_figures_of_merit(self, individual: List[int]) -> dict:
+    def calculate_figures_of_merit(self, individual: List[int], fom_dct_init: dict=None) -> dict:
         """Method calculating figures of merit with scripts or notebooks (`papermill` and scrapbook packages).
 
         :param individual: a list of 0s and 1s representing an individual to be executed
@@ -542,7 +564,7 @@ class RoxieGeneticOptimization(GeneticOptimization):
         decoded_chromosome = self.decode_individual(individual)
         global_design_variables = {key: value for (key, value) in decoded_chromosome.items() if ':' not in key}
 
-        fom_dct = {}
+        fom_dct = fom_dct_init
 
         for notebook_config in self.config.notebooks:
             notebook_folder = notebook_config.notebook_folder
@@ -589,3 +611,146 @@ class RoxieGeneticOptimization(GeneticOptimization):
             elif source in global_design_variables.keys():
                 parameters_dct[dest] = global_design_variables[source]
         return parameters_dct
+
+
+class ParallelRoxieGeneticOptimization(RoxieGeneticOptimization):
+
+    @staticmethod
+    def initialize_config(json_path: str) -> ParallelOptimizationConfig:
+        return ParallelOptimizationConfig.initialize_config(json_path)
+
+    def initialize_optimization(self):
+        root_dir = os.path.join(self.output_subdirectory_dir, self.config.ansys.root_dir)
+        template_rel_dir = os.path.join(root_dir, 'template')
+        template_files = ['15T_mech.template',
+                          '15T_mech_post_roxie.template',
+                          '15T_mech_solu.template',
+                          '15T_bc.template',
+                          '15T_Coil_geo.template',
+                          '15T_contact_el_m.template',
+                          '15T_contact_mesh.template',
+                          '15T_mat_and_elem.template',
+                          '15T_geometry_main.template',
+                          '15T_Yoke_geo.template',
+                          'CoilBlockMacro_Roxie.mac',
+                          'ContactPropMacro.mac']
+
+        input_files = ['15T_mech_%d.inp',
+                       '15T_mech_post_roxie_%d.inp',
+                       '15T_mech_solu_%d.inp',
+                       '15T_bc_%d.inp',
+                       '15T_Coil_geo_%d.inp',
+                       '15T_contact_el_m_%d.inp',
+                       '15T_contact_mesh_%d.inp',
+                       '15T_mat_and_elem_%d.inp',
+                       '15T_geometry_main_%d.inp',
+                       '15T_Yoke_geo_%d.inp',
+                       'CoilBlockMacro_Roxie_%d.mac',
+                       'ContactPropMacro_%d.mac']
+
+        for index in range(self.n_pop):
+            for template_file, input_file in zip(template_files, input_files):
+                template_path = os.path.join(template_rel_dir, template_file)
+                AnsysInputBuilder.update_input_template(template_path, index, root_dir, input_file)
+
+    def optimize(self):
+        # initial population of random chromosomes
+        self.pop = self.initialize_population()
+
+        self.initialize_optimization()
+
+        # enumerate generations
+        for gen in range(self.n_gen):
+            print('Generation:', gen)
+
+            # decode population
+            fom_dcts = []
+            for index, individual in enumerate(self.pop):
+                print('\tIndividual:', index)
+                try:
+                    self.update_model(individual)
+                    fom_dct = self.calculate_figures_of_merit(individual, {"index": index})
+                except AttributeError:
+                    fom_dct = {objective_config.objective: float('nan') for objective_config in self.config.objectives}
+
+                fom_dcts.append(fom_dct)
+
+            # execute ANSYS in parallel
+            root_dir = os.path.join(self.output_subdirectory_dir, self.config.ansys.root_dir)
+            AnsysToolAdapter.remove_ansys_output_files(root_dir, self.n_pop)
+
+            indices = self.extract_calculated_indices(fom_dcts)
+            print('indices to process: ', indices)
+            pool = LocalMapdlPool(self.config.ansys.n_parallel_runners,
+                                  exec_file=self.config.ansys.exec_file,
+                                  nproc=self.config.ansys.n_proc,
+                                  run_location=self.config.ansys.run_dir)
+            execute_ansys_in_parallel(pool, root_dir, indices)
+            pool.exit()
+
+            # read figures of merit
+            ansys_fom_dcts = AnsysToolAdapter.read_multiple_ansys_figures_of_merit(root_dir, self.n_pop)
+
+            # merge list of figures of merit
+            for i in range(self.n_pop):
+                fom_dcts[i] = {**fom_dcts[i], **ansys_fom_dcts[i]}
+
+            # calculate scores
+            scores = self.calculate_scores(fom_dcts)
+
+            # log current optimization results
+            for individual, fom_dct, score in zip(self.pop, fom_dcts, scores):
+                self.logger.append_to_logger(self.decode_individual(individual), fom_dct, score)
+
+            # display an update
+            print(sorted(scores))
+            min_logger_df = self.get_min_fitness_per_generation()
+            display(min_logger_df[min_logger_df.index == min_logger_df.index[-1]])
+
+            # apply genetic operators of selection, crossover, and mutation
+            self.pop = self.update_generation(self.pop, scores)
+            self.logger.save_logger()
+
+    def extract_calculated_indices(self, fom_dcts):
+        indices = []
+        for index, fom_dct in enumerate(fom_dcts):
+            if all([not np.isnan(value) for value in fom_dct.values()]):
+                indices.append(index)
+        return indices
+
+
+def execute_ansys_in_parallel(pool, root_dir, indices):
+    def mapping_function(mapdl, root_dir, index):
+        mapdl.clear()
+        # upload input and macro files prior to model execution
+        input_file = '15T_mech_%d.inp' % index
+        upload_files = [input_file,
+                        'Model_%d.inp' % index,
+                        'forces_edit_%d.vallone' % index,
+                        '15T_mech_post_roxie_%d.inp' % index,
+                        '15T_mech_solu_%d.inp' % index,
+                        '15T_mat_and_elem_%d.inp' % index,
+                        '15T_geometry_main_%d.inp' % index,
+                        '15T_Coil_geo_%d.inp' % index,
+                        '15T_Yoke_geo_%d.inp' % index,
+                        '15T_contact_el_m_%d.inp' % index,
+                        '15T_contact_mesh_%d.inp' % index,
+                        '15T_bc_%d.inp' % index,
+                        'ContactPropMacro_%d.mac' % index,
+                        'CoilBlockMacro_Roxie_%d.mac' % index
+                        ]
+
+        for upload_file in upload_files:
+            mapdl.upload(os.path.join(root_dir, upload_file), progress_bar=False)
+
+        # run model
+        mapdl.input(os.path.join(root_dir, input_file))
+
+        # download output file
+        output_file = 'vallone_%d.out' % index
+        mapdl.download(output_file, os.path.join(root_dir, output_file))
+        return mapdl.parameters.routine
+
+    inputs = [(root_dir, index) for index in indices]
+
+    pool.map(mapping_function, inputs, progress_bar=True, wait=True)
