@@ -257,7 +257,8 @@ class TestCableDatabase(TestCase):
         with self.assertRaises(KeyError) as context:
             self.cadata.get_cable_definition(condname)
 
-        self.assertTrue('Conductor name WRONG_COND_NAME not present in conductor definitions.' in str(context.exception))
+        self.assertTrue('Conductor name WRONG_COND_NAME not present in conductor definitions.'
+                        in str(context.exception))
 
     def test_get_cable_definition_for_condname(self):
         # arrange
