@@ -3,6 +3,7 @@ import pytest
 
 from magnumapi.geometry.GeometryFactory import GeometryFactory
 from magnumapi.cadata.CableDatabase import CableDatabase
+import magnumapi.commons.json_file as json_file
 from tests.resource_files import create_resources_file_path
 
 
@@ -19,7 +20,7 @@ def test_to_dict():
 
     # assert
     json_ref_path = create_resources_file_path('resources/geometry/roxie/16T/16T_abs.json')
-    block_layer_ref_defs = GeometryFactory.read_json_file(json_ref_path)
+    block_layer_ref_defs = json_file.read(json_ref_path)
 
     pd.testing.assert_frame_equal(pd.DataFrame(block_layer_ref_defs['block_defs']).sort_index(axis=1),
                                   pd.DataFrame(block_layer_defs['block_defs']).sort_index(axis=1))
@@ -41,7 +42,7 @@ def test_to_abs_geometry():
 
     # assert
     json_ref_path = create_resources_file_path('resources/geometry/roxie/16T/16T_abs.json')
-    block_layer_ref_defs = GeometryFactory.read_json_file(json_ref_path)
+    block_layer_ref_defs = json_file.read(json_ref_path)
 
     pd.testing.assert_frame_equal(pd.DataFrame(block_layer_ref_defs['block_defs']).sort_index(axis=1),
                                   pd.DataFrame(block_layer_defs['block_defs']).sort_index(axis=1))
@@ -63,7 +64,7 @@ def test_to_rel_dict():
 
     # assert
     json_ref_path = create_resources_file_path('resources/geometry/roxie/16T/16T_rel.json')
-    block_layer_ref_defs = GeometryFactory.read_json_file(json_ref_path)
+    block_layer_ref_defs = json_file.read(json_ref_path)
 
     pd.testing.assert_frame_equal(pd.DataFrame(block_layer_ref_defs['block_defs']).sort_index(axis=1),
                                   pd.DataFrame(block_layer_defs['block_defs']).sort_index(axis=1))
