@@ -2,9 +2,13 @@ from unittest import TestCase
 
 import pandas as pd
 
+from magnumapi.cadata.CableDatabase import CableDatabase
+from magnumapi.geometry.GeometryFactory import GeometryFactory
+from magnumapi.optimization.GeneticOptimization import RoxieGeneticOptimization
 from magnumapi.optimization.Logger import Logger
 from magnumapi.optimization.OptimizationCockpitWidget import prepare_fractions_for_bar_plot, \
-    initialize_design_variables, create_objective_table, transpose_objective_table, prepare_hover_texts_for_bar_plot
+    initialize_design_variables, create_objective_table, transpose_objective_table, prepare_hover_texts_for_bar_plot, \
+    OptimizationCockpitWidget
 from magnumapi.optimization.OptimizationConfig import OptimizationConfig
 from tests.resource_files import create_resources_file_path
 
@@ -29,10 +33,10 @@ class TestOptimizationCockpitWidget(TestCase):
         design_variables = initialize_design_variables(self.dv_df)
 
         # assert
-        self.assertEqual('2:phi_r', design_variables[0].variable_name)
+        self.assertEqual('phi_r:1:2', design_variables[0].variable_name)
         self.assertEqual('float', design_variables[0].variable_type)
         self.assertEqual('phi_r', design_variables[0].variable)
-        self.assertEqual('2:phi_r', design_variables[0].logger_column)
+        self.assertEqual('phi_r:1:2', design_variables[0].logger_column)
         self.assertAlmostEqual(3.0, design_variables[0].xl)
         self.assertAlmostEqual(10.0, design_variables[0].xu)
         self.assertEqual(6, design_variables[0].bits)
@@ -79,10 +83,10 @@ class TestOptimizationCockpitWidget(TestCase):
         fractions = prepare_fractions_for_bar_plot(design_variables, self.min_logger_df, index_act)
 
         # assert
-        fractions_ref = [0.671875, 0.390625, 0.65625, 0.671875, 0.390625, 0.671875, 0.390625, 0.671875, 0.015625,
-                         0.453125, 0.421875, 0.015625, 0.453125, 0.015625, 0.453125, 0.015625, 0.0, 0.8571428571428571,
-                         1.0, 0.3333333333333333, -0.42857142857142855, -0.14285714285714285, 1.0, -1.8571428571428572,
-                         -0.14285714285714285, 1.0, -9.0, -1.0, 1.8125, 0.587890625]
+        fractions_ref = [0.671875, 0.390625, 0.65625, 0.875, 0.25, 0.796875, 0.8125, 0.421875, 0.015625, 0.453125,
+                         0.421875, 0.296875, 0.84375, 0.375, 0.359375, 0.171875, 0.0, 0.8571428571428571, 1.0,
+                         0.3333333333333333, 0.42857142857142855, 0.0, 0.0, 1.0, 0.5714285714285714, 0.6666666666666666,
+                         1.0, 1.0, 1.8125, 0.587890625]
         self.assertListEqual(fractions_ref, fractions)
 
     def test_prepare_hover_texts_for_bar_plot(self):
@@ -95,17 +99,43 @@ class TestOptimizationCockpitWidget(TestCase):
 
         # assert
         hover_texts_ref = ['value: 7.703, range: [3.000, 10.000]', 'value: 5.734, range: [3.000, 10.000]',
-                           'value: 7.594, range: [3.000, 10.000]', 'value: 7.703, range: [3.000, 10.000]',
-                           'value: 5.734, range: [3.000, 10.000]', 'value: 7.703, range: [3.000, 10.000]',
-                           'value: 5.734, range: [3.000, 10.000]', 'value: 7.703, range: [3.000, 10.000]',
+                           'value: 7.594, range: [3.000, 10.000]', 'value: 9.125, range: [3.000, 10.000]',
+                           'value: 4.750, range: [3.000, 10.000]', 'value: 8.578, range: [3.000, 10.000]',
+                           'value: 8.688, range: [3.000, 10.000]', 'value: 5.953, range: [3.000, 10.000]',
                            'value: 0.156, range: [0.000, 10.000]', 'value: 4.531, range: [0.000, 10.000]',
-                           'value: 4.219, range: [0.000, 10.000]', 'value: 0.156, range: [0.000, 10.000]',
-                           'value: 4.531, range: [0.000, 10.000]', 'value: 0.156, range: [0.000, 10.000]',
-                           'value: 4.531, range: [0.000, 10.000]', 'value: 0.156, range: [0.000, 10.000]',
+                           'value: 4.219, range: [0.000, 10.000]', 'value: 2.969, range: [0.000, 10.000]',
+                           'value: 8.438, range: [0.000, 10.000]', 'value: 3.750, range: [0.000, 10.000]',
+                           'value: 3.594, range: [0.000, 10.000]', 'value: 1.719, range: [0.000, 10.000]',
                            'value: 0, range: [0, 7]', 'value: 6, range: [0, 7]', 'value: 3, range: [0, 3]',
-                           'value: 1, range: [0, 3]', 'value: 0, range: [3, 10]', 'value: 6, range: [7, 14]',
-                           'value: 3, range: [0, 3]', 'value: 0, range: [13, 20]', 'value: 6, range: [7, 14]',
-                           'value: 3, range: [0, 3]', 'value: 0, range: [27, 30]', 'value: 6, range: [9, 12]',
+                           'value: 1, range: [0, 3]', 'value: 6, range: [3, 10]', 'value: 7, range: [7, 14]',
+                           'value: 0, range: [0, 3]', 'value: 20, range: [13, 20]', 'value: 11, range: [7, 14]',
+                           'value: 2, range: [0, 3]', 'value: 30, range: [27, 30]', 'value: 12, range: [9, 12]',
                            'value: 1.453, range: [1.000, 1.250]', 'value: 13175.781, range: [12000.000, 14000.000]']
 
         self.assertListEqual(hover_texts_ref, hover_texts)
+
+    def test_build_programmable_geometry(self):
+        model_input_path = create_resources_file_path('resources/geometry/roxie/16T/16T_rel_slotted.json')
+        cadata_abs_path = create_resources_file_path('resources/geometry/roxie/16T/roxieold_2.cadata')
+        cadata = CableDatabase.read_cadata(cadata_abs_path)
+        geometry = GeometryFactory.init_slotted_with_json(model_input_path, cadata)
+
+        path_str = 'resources/optimization/design_variables_programmable_geometry.csv'
+        csv_global_file_path = create_resources_file_path(path_str)
+        design_variables_df = pd.read_csv(csv_global_file_path)
+
+        json_path = create_resources_file_path('resources/optimization/config.json')
+        config = OptimizationConfig.initialize_config(json_path)
+
+        prefix = 'resources/optimization/'
+        logger_path = create_resources_file_path('%sGeneticOptimizationProgrammableGeometry.csv' % prefix)
+        logger_df = pd.read_csv(logger_path)
+
+        roxie_gen_opt = RoxieGeneticOptimization(config=config,
+                                                 design_variables_df=design_variables_df,
+                                                 geometry=geometry,
+                                                 model_input_path=model_input_path,
+                                                 is_script_executed=True,
+                                                 output_subdirectory_dir='')
+        widget = OptimizationCockpitWidget(roxie_gen_opt, logger_df, config, design_variables_df)
+        widget.build()
